import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)
from loguru import logger

from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.utils.data_frame_util as data_frame_util
import pandas as pd
from datetime import datetime
import akshare as ak
import mns_scheduler.concept.ths.common.ths_concept_sync_common_api as ths_concept_sync_common_api
import mns_common.api.ths.ths_stock_api as ths_stock_api

mongodb_util = MongodbUtil('27017')


# 与概念接口对比 找出缺失数据和名称不一样的数据

def sync_new_ths_concept_by_ak_api():
    stock_board_concept_name_ths_df = ak.stock_board_concept_name_ths()
    stock_board_concept_name_ths_df = stock_board_concept_name_ths_df.rename(columns={"日期": "str_day",
                                                                                      "概念名称": "concept_name",
                                                                                      "成分股数量": "numbers",
                                                                                      "网址": "url",
                                                                                      "代码": "concept_code",
                                                                                      })

    stock_board_concept_name_ths_df['str_day'].fillna(datetime(1970, 1, 1), inplace=True)
    stock_board_concept_name_ths_df.concept_name = stock_board_concept_name_ths_df.concept_name.str.replace('（', '(')
    stock_board_concept_name_ths_df.concept_name = stock_board_concept_name_ths_df.concept_name.str.replace('）', ')')

    stock_board_concept_name_ths_df['str_day'] = stock_board_concept_name_ths_df['str_day'].apply(
        lambda x: datetime.strftime(x, '%Y-%m-%d'))

    # 同花顺概念列表
    ths_concept_list_exist = mongodb_util.find_all_data('ths_concept_list')

    for concept_one in stock_board_concept_name_ths_df.itertuples():
        try:
            exist_concept_df_one = ths_concept_list_exist.loc[
                (ths_concept_list_exist['name'] == concept_one.concept_name)
                | (ths_concept_list_exist['web_concept_code'] == concept_one.concept_code)]
            now_date = datetime.now()
            str_now_time = now_date.strftime('%Y-%m-%d %H:%M:%S')
            str_day = concept_one.str_day
            if data_frame_util.is_empty(exist_concept_df_one):
                concept_code = concept_one.concept_code

                diff_one = {
                    '_id': int(concept_code),
                    'symbol': int(concept_code),
                    'name': concept_one.concept_name,
                    'url': concept_one.url,
                    'str_day': str_day,
                    'success': True,
                    'str_now_time': str_now_time,
                    'web_concept_code': int(concept_one.concept_code),
                    'web_concept_url': concept_one.url,
                    'valid': True
                }
                diff_one_df = pd.DataFrame(diff_one, index=[1])
                mongodb_util.save_mongo(diff_one_df, 'ths_concept_list')
                url = 'http://q.10jqka.com.cn/gn/detail/code/' + str(concept_one.concept_code)
                # 推送新概念信息到微信
                ths_concept_sync_common_api.push_msg_to_we_chat_web(concept_code, concept_one.concept_name,
                                                                    url)
                new_concept_symbol_df = get_concept_detail_info_web(concept_code)
                if new_concept_symbol_df is None or new_concept_symbol_df.shape[0] == 0:
                    return None
                new_concept_symbol_df.loc[:, 'way'] = 'index_sync'
                ths_concept_sync_common_api.save_ths_concept_detail(new_concept_symbol_df, concept_one.concept_name,
                                                                    str_day,
                                                                    str_now_time, concept_code)
                logger.info("新增同花顺新概念:{}", concept_one.concept_name)
        except BaseException as e:
            logger.error("同步概念:{},信息异常:{}", concept_one.concept_name, e)


# 获取web端 新增概念详情
def get_concept_detail_info_web(concept_code):
    new_concept_symbol_list = ths_stock_api.ths_stock_concept(concept_code)
    if new_concept_symbol_list is None or new_concept_symbol_list.shape[0] == 0:
        return None
    new_concept_symbol_list = new_concept_symbol_list[ths_concept_sync_common_api.order_fields]
    new_concept_symbol_list['_id'] = str(concept_code) + '-' + new_concept_symbol_list['symbol']
    return new_concept_symbol_list


if __name__ == '__main__':
    get_concept_detail_info_web(886068)
    sync_new_ths_concept_by_ak_api()



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)
import mns_common.api.ths.ths_stock_api as ths_stock_api
import datetime
import time
import mns_common.api.msg.push_msg_api as push_msg_api
from loguru import logger
import mns_scheduler.concept.ths.common.ths_concept_sync_common_api as ths_concept_sync_common_api
import pandas as pd
from mns_common.db.MongodbUtil import MongodbUtil

mongodb_util = MongodbUtil('27017')


# app 端获取新增概念信息 接口不稳定

# 获取最大概念代码
def get_max_concept_name():
    query = {"symbol": {'$ne': 'null'}, "success": True}
    return mongodb_util.descend_query(query, 'ths_concept_list', 'symbol', 1)


# 获取app端新概念名称
def get_concept_detail_info_app(concept_code):
    return ths_stock_api.get_new_concept_code_app(concept_code)


# 获取app端新概念详细列表
def get_new_concept_detail_list_app(concept_code):
    new_concept_symbol_list = ths_stock_api.get_new_concept_list_app(concept_code)
    if new_concept_symbol_list is None or new_concept_symbol_list.shape == 0:
        return None
    new_concept_symbol_list = new_concept_symbol_list[ths_concept_sync_common_api.order_fields]
    new_concept_symbol_list['_id'] = str(concept_code) + '-' + new_concept_symbol_list['symbol']
    return new_concept_symbol_list


# 推送新概念信息到微信
def push_msg_to_we_chat_app(concept_code, concept_name):
    url = f'https://m.10jqka.com.cn/hq/industry.html#code={concept_code}&refCountId=R_55ba2964_186'
    msg = "概念代码:" + str(concept_code) + "," + "概念名称:" + concept_name + "," + "url:   " + url
    title = "新增概念:" + str(concept_code) + "-" + concept_name
    push_msg_api.push_msg_to_wechat(title, msg)


# 保存新概念信息到数据库
def save_ths_concept_list_app(concept_code, concept_name, str_day, str_now_time, success):
    concept_code = int(concept_code)
    url = 'https://m.10jqka.com.cn/hq/industry.html#code=' + str(concept_code) + '&refCountId=R_55ba2964_186'
    ths_concept_list = pd.DataFrame([
        [concept_code, concept_code, concept_name, str_day, url, str_now_time, success],
    ], columns=['_id', 'symbol', 'name', 'str_day', 'url', 'str_now_time', 'success'])

    mongodb_util.save_mongo(ths_concept_list, 'ths_concept_list')


# 同步新概念信息 从app爬取
def sync_new_concept_data():
    max_concept_name = get_max_concept_name()
    try:
        now_date = datetime.datetime.now()
        str_day = now_date.strftime('%Y-%m-%d')
        str_now_time = now_date.strftime('%Y-%m-%d %H:%M:%S')
        concept_code = list(max_concept_name['symbol'])[0]
        success = list(max_concept_name['success'])[0]
        if success:
            concept_code = concept_code + 1

        new_concept_df = get_concept_detail_info_app(concept_code)
        if new_concept_df is None or new_concept_df.shape[0] == 0:
            time.sleep(1)
            return None

        concept_name = list(new_concept_df['concept_name'])[0]
        concept_code = list(new_concept_df['concept_code'])[0]
        if success:
            # 推送新概念信息到微信
            push_msg_to_we_chat_app(concept_code, concept_name)
        # 获取新概念详细信息
        new_concept_symbol_list = get_new_concept_detail_list_app(concept_code)

        if new_concept_symbol_list is None or new_concept_symbol_list.shape[0] == 0:
            # 保存新概念信息到数据库
            query = {"symbol": int(concept_code), "success": True}
            exist_success = mongodb_util.exist_data_query('ths_concept_list', query)
            if bool(1 - exist_success):
                save_ths_concept_list_app(concept_code, concept_name, str_day, str_now_time, False)
                time.sleep(1)

        else:
            # 保存新概念信息到数据库
            save_ths_concept_list_app(concept_code, concept_name, str_day, str_now_time, True)
            new_concept_symbol_list.loc[:, 'way'] = 'index_sync'
            # 保存新概念详细信息到数据库
            ths_concept_sync_common_api.save_ths_concept_detail(new_concept_symbol_list, concept_name, str_day,
                                                                str_now_time, concept_code)
            time.sleep(1)

    except BaseException as e:
        logger.error("同步新概念异常:{},concept_code:{}", e, concept_code)


if __name__ == '__main__':
    # myquery1 = {"symbol": {"$ne": 'null'}}
    # newvalues1 = {"$set": {"success": True}}
    #
    # mongodb_util_remote.update_many(myquery1, newvalues1, 'ths_concept_list')
    df = get_new_concept_detail_list_app('886033')
    sync_new_concept_data()
    # code = 886026
    # while True:
    #     symbol_new_concept = get_concept_detail_info_app(code)
    #     logger.info(symbol_new_concept)
    #     new_concept_symbols = ths_stock_api.get_new_concept_list_app(code)
    #     logger.info(new_concept_symbols)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)

# 股权质押 https://data.eastmoney.com/gpzy/pledgeRatio.aspx

import requests


def get_equity_mortgage():
    url = (
        'https://datacenter-web.eastmoney.com/api/data/v1/get?callback=jQuery1123018114616341595302_1715993497168&sortColumns=PLEDGE_RATIO&sortTypes=-1'
        '&pageSize=50000'
        '&pageNumber=1'
        '&reportName=RPT_CSDC_LIST&columns=ALL&quoteColumns=&source=WEB'
        '&client=WEB'
        '&filter=(TRADE_DATE%3D%272024-05-17%27)')

    headers = {
        "Content-Type": "application/json"
    }
    r = requests.post(url, headers=headers)
    data_json = r.json()
    print(data_json)


if __name__ == '__main__':
    get_equity_mortgage()



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)

import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)

import pandas as pd
from loguru import logger
import mns_common.component.common_service_fun_api as common_service_fun_api
import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api
import mns_common.api.em.east_money_stock_api as east_money_stock_api
import mns_common.utils.date_handle_util as date_handle_util
import mns_common.component.company.company_common_service_api as company_common_service_api
import mns_common.component.data.data_init_api as data_init_api
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.utils.data_frame_util as data_frame_util
import mns_common.component.zt.zt_common_service_api as zt_common_service_api

mongodb_util = MongodbUtil('27017')
mongodb_util_27019 = MongodbUtil('27019')

save_zt_name = 'realtime_quotes_now_zt_new'
choose_field = ["_id",
                "symbol",
                "name",
                "industry",
                "now_price",
                "chg",
                "quantity_ratio",
                "amount_level",
                "disk_ratio",
                "real_disk_diff_amount_exchange",
                'max_real_main_inflow_multiple',
                'sum_main_inflow_disk',
                "real_main_inflow_multiple",
                "real_super_main_inflow_multiple",
                "super_main_inflow_multiple",
                "main_inflow_multiple",
                "disk_diff_amount_exchange",
                "large_inflow_multiple",
                "today_main_net_inflow",
                "today_main_net_inflow_ratio",
                "super_large_order_net_inflow",
                "super_large_order_net_inflow_ratio",
                "large_order_net_inflow",
                "large_order_net_inflow_ratio",
                "reference_main_inflow",
                "disk_diff_amount",
                "mv_circulation_ratio",
                "real_exchange",
                "exchange",
                'real_exchange',
                "total_mv",
                "flow_mv",
                "volume",
                "high",
                "low",
                "open",
                "yesterday_price",
                "amount",
                "total_mv_sp",
                "flow_mv_sp",
                "outer_disk",
                "inner_disk",
                "classification",
                "number",
                "str_day",
                "str_now_date"
                ]


def create_index():
    mongodb_util.create_index(save_zt_name,
                              [("symbol", 1), ("number", 1)])

    mongodb_util.create_index(save_zt_name,
                              [("symbol", 1)])

    mongodb_util.create_index(save_zt_name,
                              [("number", 1)])

    mongodb_util.create_index(save_zt_name,
                              [("str_day", 1)])

    mongodb_util.create_index(save_zt_name,
                              [("str_now_date", 1)])
    mongodb_util.create_index(save_zt_name,
                              [("symbol", 1), ("str_now_date", 1)])
    mongodb_util.create_index(save_zt_name,
                              [("str_day", 1), ("str_now_date", 1)])

    mongodb_util.create_index(save_zt_name,
                              [("symbol", 1), ("str_day", 1)])


def get_mongodb_util(str_day):
    tag = mongodb_util.exist_data_query('trade_date_list', query={'_id': str_day, "tag": True})
    if tag:
        return mongodb_util_27019
    else:
        return mongodb_util


# 同步一天高涨幅列表
def save_stock_high_chg_pool(str_day):
    real_time_quotes_now_init = east_money_stock_api.get_real_time_quotes_all_stocks()

    sync_date = date_handle_util.add_date_day(date_handle_util.no_slash_date(str_day), 0)
    str_now_day = sync_date.strftime('%Y-%m-%d')

    realtime_quotes_db_name = 'realtime_quotes_now_' + str_now_day
    number = get_today_realtime_quotes_max_number(realtime_quotes_db_name, str_day)

    query = {"number": number, "chg": {'$gte': 9.7}}

    real_time_quotes_now = mongodb_util.find_query_data(realtime_quotes_db_name, query)
    if real_time_quotes_now.shape[0] == 0:
        return
    real_time_quotes_now = common_service_fun_api.total_mv_classification(
        real_time_quotes_now.copy())
    real_time_quotes_now = real_time_quotes_now.sort_values(by=['chg'], ascending=False)
    real_time_quotes_now = common_service_fun_api.exclude_new_stock(real_time_quotes_now)

    # todo fix industry
    industry_group_df = company_common_service_api.get_company_info_industry()
    industry_group_df = industry_group_df.set_index(['_id'], drop=True)
    real_time_quotes_now.drop(columns=['industry'], inplace=True)
    real_time_quotes_now = real_time_quotes_now.set_index(['symbol'], drop=False)
    real_time_quotes_now = pd.merge(real_time_quotes_now, industry_group_df, how='outer',
                                    left_index=True, right_index=True)
    real_time_quotes_now = real_time_quotes_now.dropna(inplace=False)

    zt_symbol_info = zt_common_service_api.get_last_trade_day_zt(str_day)

    zt_symbol_connected_boards_numbers_list = zt_symbol_info.loc[
        zt_symbol_info['symbol'].isin(real_time_quotes_now['symbol'])]

    real_time_quotes_now['connected_boards_numbers'] = 1
    if zt_symbol_connected_boards_numbers_list is not None and zt_symbol_connected_boards_numbers_list.shape[0] != 0:
        for yesterday_zt in zt_symbol_connected_boards_numbers_list.itertuples():
            real_time_quotes_now.loc[
                real_time_quotes_now["symbol"] == yesterday_zt.symbol, ['connected_boards_numbers']] \
                = 1 + yesterday_zt.connected_boards_numbers

    for stock_one in real_time_quotes_now.itertuples():
        try:

            symbol = stock_one.symbol

            classification = common_service_fun_api.classify_symbol_one(symbol)

            chg = float(stock_one.chg)
            if classification == 'X' and chg > 30:
                continue
            elif classification != 'X' and chg > 21:
                continue
            real_time_quotes_now_one = real_time_quotes_now_init.loc[
                real_time_quotes_now_init['symbol'] == symbol]
            list_date = list(real_time_quotes_now_one['list_date'])[0]
            list_date = str(list_date)
            list_date = list_date.replace(".0", "")
            list_date = date_handle_util.lash_date(list_date)

            list_date_time = date_handle_util.str_to_date(list_date, "%Y-%m-%d")
            str_now_day_time = date_handle_util.str_to_date(str_now_day, "%Y-%m-%d")

            list_to_now_days = date_handle_util.days_diff(str_now_day_time, list_date_time)

            last_trade_day = trade_date_common_service_api.get_last_trade_day(str_day)

            last_day_high_chg = mongodb_util.exist_data_query('stock_hfq_daily',
                                                              query={"symbol": symbol, "date": last_trade_day,
                                                                     "chg": {'$gte': 9}})

            open = stock_one.open
            high = stock_one.high
            close = stock_one.now_price
            yi_zhi_ban = False
            if open == high and open == close:
                yi_zhi_ban = True

            stock_high_chg_pool = {
                '_id': symbol + '_' + str_now_day,
                'symbol': symbol,
                'name': stock_one.name,
                'industry': stock_one.industry,
                'str_day': str_now_day,
                'classification': classification,
                'chg': stock_one.chg,
                "connected_boards_numbers": stock_one.connected_boards_numbers,
                "open": stock_one.open,
                "close": stock_one.now_price,
                "high": stock_one.high,
                "low": stock_one.low,
                "volume": stock_one.volume,
                "amount": stock_one.amount,
                "flow_mv_level": stock_one.flow_mv_level,
                "flow_mv_sp": stock_one.flow_mv_sp,
                "total_mv_sp": stock_one.total_mv_sp,
                "list_to_now_days": list_to_now_days,
                "list_date": list_date,
                "exchange": stock_one.exchange,
                "last_day_high_chg": last_day_high_chg,
                "yi_zhi_ban": yi_zhi_ban
            }
            mongodb_util.save_mongo(pd.DataFrame([stock_high_chg_pool], index=[0]),
                                    'stock_high_chg_pool')
            logger.info("更新高涨幅数据成功{},{}", stock_one.symbol, str_now_day)
        except Exception as e:
            logger.error("更新高涨幅数据异常:{},{},{}", stock_one.symbol, str_now_day, e)
            continue


def get_today_realtime_quotes_max_number(realtime_quotes_db_name, str_day):
    mongo = get_mongodb_util(str_day)
    query = {"symbol": "000001"}
    realtime_quotes = mongo.descend_query(query, realtime_quotes_db_name, 'number', 1)
    if realtime_quotes is None or realtime_quotes.shape[0] == 0:
        return 1
    else:
        return list(realtime_quotes['number'])[0]


def get_high_chg_symbol(str_day):
    realtime_quotes_db_name = 'stock_qfq_daily'

    query = {"date": date_handle_util.no_slash_date(str_day), "chg": {'$gte': 9.7}}
    # 今日高涨幅的list
    real_time_quotes_now_high_chg = mongodb_util.find_query_data(realtime_quotes_db_name, query)
    if real_time_quotes_now_high_chg.shape[0] == 0:
        return None
    high_chg_list = list(real_time_quotes_now_high_chg['symbol'])
    # 今日涨停股

    query_zt = {'str_day': str_day}
    zt_pool = mongodb_util.find_query_data('stock_zt_pool', query_zt)
    if zt_pool.shape[0] > 0:
        zt_pool_list = list(zt_pool['symbol'])
        high_chg_list.extend(zt_pool_list)
    high_chg_list = list(set(high_chg_list))
    return high_chg_list


# 保存一天详细涨停数据
def sync_his_high_one_day(str_day):
    mongo = get_mongodb_util(str_day)

    realtime_quotes_db_name = 'realtime_quotes_now_' + str_day
    high_chg_list = get_high_chg_symbol(str_day)
    if high_chg_list is None or len(high_chg_list) == 0:
        return

    for symbol in high_chg_list:
        try:
            query_all = {"symbol": symbol}
            real_time_quotes_now_high_chg_all = mongo.find_query_data(realtime_quotes_db_name, query_all)
            if real_time_quotes_now_high_chg_all.shape[0] == 0:
                return
            real_time_quotes_now_high_chg_all = company_common_service_api.amendment_industry_exist_na(
                real_time_quotes_now_high_chg_all,
                high_chg_list)
            real_time_quotes_now_high_chg_all.dropna(subset=['symbol'], axis=0,
                                                     inplace=True)
            real_time_quotes_now_high_chg_all = data_init_api.calculate_parameter_factor(
                real_time_quotes_now_high_chg_all)

            real_time_quotes_now_high_chg_all['amount_level'] = round(
                (real_time_quotes_now_high_chg_all['amount'] / common_service_fun_api.HUNDRED_MILLION), 2) * 10
            real_time_quotes_now_high_chg_all['flow_mv_sp'] = round(
                (real_time_quotes_now_high_chg_all['flow_mv'] / common_service_fun_api.HUNDRED_MILLION), 2)
            real_time_quotes_now_high_chg_all['total_mv_sp'] = round(
                (real_time_quotes_now_high_chg_all['total_mv'] / common_service_fun_api.HUNDRED_MILLION), 2)

            save_realtime_quotes_now_zt_data(real_time_quotes_now_high_chg_all, str_day, symbol)
            logger.info("同步涨停信息:{},{}", str_day, symbol)
        except BaseException as e:
            logger.error("发生异常:{}:{},{}", str_day, e, symbol)


def save_one_symbol_day(symbol, str_day):
    mongo = get_mongodb_util(str_day)
    realtime_quotes_db_name = 'realtime_quotes_now_' + str_day
    high_chg_list = [symbol]
    query_all = {"symbol": symbol}
    real_time_quotes_now_high_chg_all = mongo.find_query_data(realtime_quotes_db_name, query_all)
    if real_time_quotes_now_high_chg_all.shape[0] == 0:
        return
    real_time_quotes_now_high_chg_all = company_common_service_api.amendment_industry_exist_na(
        real_time_quotes_now_high_chg_all,
        high_chg_list)
    real_time_quotes_now_high_chg_all = real_time_quotes_now_high_chg_all.dropna(inplace=False)
    real_time_quotes_now_high_chg_all = data_init_api.calculate_parameter_factor(
        real_time_quotes_now_high_chg_all)

    real_time_quotes_now_high_chg_all.loc[:, 'amount_level'] = round(
        (real_time_quotes_now_high_chg_all.loc[:, 'amount'] / common_service_fun_api.HUNDRED_MILLION), 2) * 10
    real_time_quotes_now_high_chg_all.loc[:, 'flow_mv_sp'] = round(
        (real_time_quotes_now_high_chg_all.loc[:, 'flow_mv'] / common_service_fun_api.HUNDRED_MILLION), 2)
    real_time_quotes_now_high_chg_all.loc[:, 'total_mv_sp'] = round(
        (real_time_quotes_now_high_chg_all['total_mv'] / common_service_fun_api.HUNDRED_MILLION), 2)

    save_realtime_quotes_now_zt_data(real_time_quotes_now_high_chg_all, str_day, symbol)


def save_realtime_quotes_now_zt_data(realtime_quotes_now_zt, str_day, symbol):
    realtime_quotes_now_zt = common_service_fun_api.classify_symbol(realtime_quotes_now_zt.copy())
    # create_index()
    if 'wei_bi' in realtime_quotes_now_zt.columns and bool(1 - ("wei_bi" in choose_field)):
        choose_field.append("wei_bi")
    if 'up_speed' in realtime_quotes_now_zt.columns and bool(1 - ("up_speed" in choose_field)):
        choose_field.append("up_speed")
    if 'list_date' in realtime_quotes_now_zt.columns and bool(1 - ("list_date" in choose_field)):
        choose_field.append("list_date")
    realtime_quotes_now_zt.loc[:, 'str_day'] = str_day
    realtime_quotes_now_zt = realtime_quotes_now_zt[choose_field]
    remove_query = {"symbol": symbol, "str_day": str_day}
    result = mongodb_util.remove_data(remove_query, save_zt_name).acknowledged
    if result:
        mongodb_util.insert_mongo(realtime_quotes_now_zt, save_zt_name)


def sync_one_day_zt_info(str_day):
    sync_his_high_one_day(str_day)
    save_stock_high_chg_pool(str_day)


def sync_all_data():
    create_index()
    end_date = date_handle_util.add_date_day('20231029', 0)

    sync_date = date_handle_util.add_date_day('20231030', 0)

    str_now_day = sync_date.strftime('%Y-%m-%d')
    while sync_date > end_date:
        try:
            sync_one_day_zt_info(str_now_day)
            sync_date = date_handle_util.add_date_day(date_handle_util.no_slash_date(str_now_day), -1)
            str_now_day = sync_date.strftime('%Y-%m-%d')
            logger.info(str_now_day)
        except BaseException as e:
            logger.error("发生异常:{}:{}", str_now_day, e)


# fix miss symbol
def stock_qfq_daily_high_chg_pool(str_day):
    high_chg_list = get_high_chg_symbol(str_day)

    realtime_quotes_db_name = 'realtime_quotes_now_' + str_day
    number = get_today_realtime_quotes_max_number(realtime_quotes_db_name, str_day)

    query = {"number": number, "chg": {'$gte': 9.7}}
    mongodb = get_mongodb_util(str_day)
    real_time_quotes_now = mongodb.find_query_data(realtime_quotes_db_name, query)
    if real_time_quotes_now.shape[0] == 0:
        high_chg_list = high_chg_list
    else:
        real_time_quotes_now_symbol_list = real_time_quotes_now['symbol']
        high_chg_list.extend(real_time_quotes_now_symbol_list)
        high_chg_list = list(set(high_chg_list))

    if high_chg_list is None:
        return

    query = {'symbol': {"$in": high_chg_list}, 'date': date_handle_util.no_slash_date(str_day)}
    stock_qfq_daily_list = mongodb_util.find_query_data('stock_qfq_daily', query)

    real_time_quotes_now_init = east_money_stock_api.get_real_time_quotes_all_stocks()

    stock_qfq_daily_list = stock_qfq_daily_list.sort_values(by=['chg'], ascending=False)

    zt_symbol_info = zt_common_service_api.get_last_trade_day_zt(str_day)

    zt_symbol_connected_boards_numbers_list = zt_symbol_info.loc[
        zt_symbol_info['symbol'].isin(stock_qfq_daily_list['symbol'])]

    stock_qfq_daily_list['connected_boards_numbers'] = 1
    if zt_symbol_connected_boards_numbers_list is not None and zt_symbol_connected_boards_numbers_list.shape[0] != 0:
        for yesterday_zt in zt_symbol_connected_boards_numbers_list.itertuples():
            stock_qfq_daily_list.loc[
                stock_qfq_daily_list["symbol"] == yesterday_zt.symbol, ['connected_boards_numbers']] \
                = 1 + yesterday_zt.connected_boards_numbers

    for stock_one in stock_qfq_daily_list.itertuples():
        try:

            symbol = stock_one.symbol

            classification = common_service_fun_api.classify_symbol_one(symbol)

            real_time_quotes_now_one = real_time_quotes_now_init.loc[
                real_time_quotes_now_init['symbol'] == symbol]
            if data_frame_util.is_empty(real_time_quotes_now_one):
                list_date = '19890729'
            else:
                list_date = list(real_time_quotes_now_one['list_date'])[0]
                list_date = str(list_date)
                list_date = list_date.replace(".0", "")
            list_date = date_handle_util.lash_date(list_date)

            list_date_time = date_handle_util.str_to_date(list_date, "%Y-%m-%d")
            str_now_day_time = date_handle_util.str_to_date(str_day, "%Y-%m-%d")

            list_to_now_days = date_handle_util.days_diff(str_now_day_time, list_date_time)
            if list_to_now_days == 0:
                # 排除当天上市新股
                continue

            last_trade_day = trade_date_common_service_api.get_last_trade_day(str_day)

            last_day_high_chg = mongodb_util.exist_data_query('stock_hfq_daily',
                                                              query={"symbol": symbol, "date": last_trade_day,
                                                                     "chg": {'$gte': 9}})

            open = stock_one.open
            high = stock_one.high
            close = stock_one.close
            yi_zhi_ban = False
            if open == high and open == close:
                yi_zhi_ban = True

            stock_high_chg_pool = {
                '_id': symbol + '_' + str_day,
                'symbol': symbol,
                'name': stock_one.name,
                'industry': stock_one.industry,
                'str_day': str_day,
                'classification': classification,
                'chg': stock_one.chg,
                "connected_boards_numbers": stock_one.connected_boards_numbers,
                "open": stock_one.open,
                "close": stock_one.close,
                "high": stock_one.high,
                "low": stock_one.low,
                "volume": stock_one.volume,
                "amount": stock_one.amount,
                # "flow_mv_level": stock_one.flow_mv_level,
                "flow_mv_sp": stock_one.flow_mv_sp,
                # "total_mv_sp": stock_one.total_mv_sp,
                "list_to_now_days": list_to_now_days,
                "list_date": list_date,
                "exchange": stock_one.exchange,
                "last_day_high_chg": last_day_high_chg,
                "yi_zhi_ban": yi_zhi_ban,
                "miss_out": True
            }
            query_exists = {"symbol": symbol, "str_day": str_day}
            if mongodb_util.exist_data_query('stock_high_chg_pool', query_exists):
                continue
            else:
                mongodb_util.save_mongo(pd.DataFrame([stock_high_chg_pool], index=[0]),
                                        'stock_high_chg_pool')
            logger.info("更新高涨幅数据成功{},{}", stock_one.symbol, str_day)
        except Exception as e:
            logger.error("更新高涨幅数据异常:{},{},{}", stock_one.symbol, str_day, e)
            continue


if __name__ == '__main__':
    sync_one_day_zt_info('2024-05-17')

    # sync_date = date_handle_util.add_date_day('20221022', 0)
    # now_date = datetime.now()
    # str_now_day = sync_date.strftime('%Y-%m-%d')
    # while now_date > sync_date:
    #     try:
    #         save_one_symbol_day('000948', str_now_day)
    #         sync_date = date_handle_util.add_date_day(date_handle_util.no_slash_date(str_now_day), 1)
    #         str_now_day = sync_date.strftime('%Y-%m-%d')
    #         print(str_now_day)
    #     except BaseException as e:
    #         logger.error("发生异常:{}:{}", str_now_day, e)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)

import mns_common.utils.date_handle_util as date_handle_util
from loguru import logger

import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api
import mns_common.component.company.company_common_service_api as company_common_service_api
import mns_common.component.common_service_fun_api as common_service_fun_api
import mns_common.component.data.data_init_api as data_init_api
import pandas as pd
from datetime import time
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.utils.data_frame_util as data_frame_util
import mns_common.component.k_line.common.k_line_common_service_api as k_line_common_service_api

mongodb_util = MongodbUtil('27017')
mongodb_util_21019 = MongodbUtil('27019')

realtime_quotes_now_zt_new_kc_open_field = ['_id',
                                            'symbol',
                                            'name',
                                            'chg',
                                            'quantity_ratio',
                                            'amount_level',
                                            'real_exchange',
                                            'sum_main_inflow_disk',
                                            'disk_diff_amount',
                                            'flow_mv_level',
                                            'total_mv_level',
                                            'today_chg',
                                            'industry',
                                            'first_sw_industry',
                                            'second_sw_industry',
                                            'third_sw_industry',
                                            'ths_concept_name',
                                            'ths_concept_code',
                                            'em_industry',
                                            'disk_ratio',
                                            'company_type',
                                            'reference_main_inflow',
                                            'main_inflow_multiple', 'super_main_inflow_multiple',
                                            'disk_diff_amount_exchange', 'exchange', 'amount', 'today_main_net_inflow',
                                            'today_main_net_inflow_ratio', 'super_large_order_net_inflow',
                                            'super_large_order_net_inflow_ratio', 'large_order_net_inflow',
                                            'large_order_net_inflow_ratio', 'now_price',
                                            'high', 'low', 'open', 'yesterday_price',
                                            'volume', 'total_mv', 'flow_mv',
                                            'outer_disk', 'inner_disk',
                                            'classification', 'str_now_date', 'number', 'str_day',
                                            'yesterday_high_chg',
                                            'ths_concept_sync_day', 'mv_circulation_ratio',
                                            'large_inflow_multiple', 'real_main_inflow_multiple',
                                            'max_real_main_inflow_multiple', 'list_date',
                                            'real_super_main_inflow_multiple', 'real_flow_mv',
                                            'real_disk_diff_amount_exchange', 'no_open_data']


# fix 错杀数据 有成交量的数据
def fix_industry_data(real_time_quotes_now):
    #  fix industry
    real_time_quotes_now_r = company_common_service_api.amendment_industry(real_time_quotes_now.copy())

    symbol_list = list(real_time_quotes_now_r['symbol'])

    na_real_now = real_time_quotes_now.loc[
        ~(real_time_quotes_now['symbol'].isin(symbol_list))]

    na_real_now = na_real_now.loc[na_real_now['amount'] != 0]

    real_time_quotes_now_result = pd.concat([real_time_quotes_now_r, na_real_now], axis=0)
    return real_time_quotes_now_result


def get_db(str_day):
    trade_date_list = mongodb_util.find_query_data('trade_date_list', query={"_id": str_day, 'tag': False})
    tag = trade_date_list.shape[0] > 0
    if tag:
        return mongodb_util
    else:
        return mongodb_util_21019


def create_db_index(db):
    db.create_index('realtime_quotes_now_zt_new_kc', [("str_day", 1)])
    db.create_index('realtime_quotes_now_zt_new_kc', [("symbol", 1)])
    db.create_index('realtime_quotes_now_zt_new_kc', [("number", 1)])
    db.create_index('realtime_quotes_now_zt_new_kc', [("symbol", 1), ("number", 1)])
    db.create_index('realtime_quotes_now_zt_new_kc', [("str_now_date", 1)])


def sync_all_kc_zt_data(str_day, symbols):
    create_db_index(mongodb_util)
    if symbols is None:
        query_daily = {'date': date_handle_util.no_slash_date(str_day),
                       "chg": {"$gte": 9.0}}
    else:
        query_daily = {'date': date_handle_util.no_slash_date(str_day),
                       'symbol': {"$in": symbols},
                       "chg": {"$gte": 9.0}}

    kc_stock_qfq_daily = mongodb_util.find_query_data('stock_qfq_daily', query_daily)
    if data_frame_util.is_empty(kc_stock_qfq_daily):
        logger.error("无k线数据:{}", symbols)
        return

    company_info = company_common_service_api.get_company_info_industry_list_date()
    company_info['symbol'] = company_info['_id']
    company_info = company_info.loc[company_info['_id'].isin(list(kc_stock_qfq_daily['symbol']))]

    for stock_one in company_info.itertuples():
        try:

            kc_stock_qfq_daily.loc[
                kc_stock_qfq_daily['symbol'] == stock_one.symbol, 'industry'] = stock_one.industry
            kc_stock_qfq_daily.loc[
                kc_stock_qfq_daily['symbol'] == stock_one.symbol, 'first_sw_industry'] = stock_one.first_sw_industry
            kc_stock_qfq_daily.loc[
                kc_stock_qfq_daily[
                    'symbol'] == stock_one.symbol, 'second_sw_industry'] = stock_one.second_sw_industry
            kc_stock_qfq_daily.loc[
                kc_stock_qfq_daily['symbol'] == stock_one.symbol, 'third_sw_industry'] = stock_one.third_sw_industry

            kc_stock_qfq_daily.loc[
                kc_stock_qfq_daily['symbol'] == stock_one.symbol, 'list_date'] = stock_one.list_date

            str_day_date = date_handle_util.str_to_date(str_day, '%Y-%m-%d')

            list_date = date_handle_util.str_to_date(str(stock_one.list_date).replace(".0", ""), '%Y%m%d')

            # 计算日期差值 距离现在上市时间
            kc_stock_qfq_daily.loc[
                kc_stock_qfq_daily['symbol'] == stock_one.symbol, 'diff_days'] = (str_day_date - list_date).days

            last_trade_day = trade_date_common_service_api.get_last_trade_day(str_day)

            query_high_chg = {'chg': {"$gte": 11}, 'date': date_handle_util.no_slash_date(last_trade_day),
                              "symbol": stock_one.symbol}
            yesterday_high_chg = mongodb_util.exist_data_query('stock_qfq_daily', query_high_chg)
            if yesterday_high_chg:
                kc_stock_qfq_daily.loc[
                    kc_stock_qfq_daily['symbol'] == stock_one.symbol, 'yesterday_high_chg'] = True
            else:
                kc_stock_qfq_daily.loc[
                    kc_stock_qfq_daily['symbol'] == stock_one.symbol, 'yesterday_high_chg'] = False

        except BaseException as e:
            logger.error("出现异常:{},{},{}", e, str_day, stock_one.symbol)

    mongodb_util.save_mongo(kc_stock_qfq_daily, 'kc_stock_qfq_daily')
    kc_stock_qfq_daily = kc_stock_qfq_daily.loc[(kc_stock_qfq_daily['classification'].isin(['K', 'C', 'X']))
                                                | (kc_stock_qfq_daily['name'].str.startswith('C'))]
    kc_stock_qfq_daily = common_service_fun_api.exclude_new_stock(kc_stock_qfq_daily)

    kc_stock_qfq_daily = kc_stock_qfq_daily.sort_values(by=['chg'], ascending=False)

    for kc_one in kc_stock_qfq_daily.itertuples():
        try:
            db_name = 'realtime_quotes_now_' + str_day
            logger.info("同步高涨幅开盘日期和代码:{},{}", str_day, kc_one.symbol)

            query_real_time = {'symbol': kc_one.symbol}
            db = get_db(str_day)
            realtime_quotes_now_kc = db.find_query_data(db_name, query_real_time)
            realtime_quotes_now_kc = common_service_fun_api.exclude_new_stock(realtime_quotes_now_kc)
            if realtime_quotes_now_kc.shape[0] == 0:
                logger.error("当期日期代码无开盘数据:{},{}", str_day, kc_one.symbol)
                continue
            # 同步当天高涨幅集合竞价数据
            one_symbol_day_open_data(realtime_quotes_now_kc, kc_one, str_day)
            # # 同步高涨幅整天交易数据
            one_symbol_day_data(str_day, kc_one, realtime_quotes_now_kc)
        except BaseException as e:
            logger.error("出现异常:{},{},{}", e, str_day, kc_one.symbol)


def one_symbol_day_open_data(realtime_quotes_now_kc, kc_one, str_day):
    realtime_quotes_now_kc = realtime_quotes_now_kc.sort_values(by=['_id'], ascending=True)
    realtime_quotes_now_kc = realtime_quotes_now_kc.loc[realtime_quotes_now_kc['str_now_date'] >= str_day + " 09:26:00"]
    realtime_quotes_now_zt_new_kc_open = realtime_quotes_now_kc.iloc[0:1]
    _id = list(realtime_quotes_now_zt_new_kc_open['_id'])[0]
    str_now_date = _id[7:26]
    now_date = date_handle_util.str_to_date(str_now_date, "%Y-%m-%d %H:%M:%S")
    now_date_time = now_date.time()
    target_time_09_31 = time(9, 31)
    realtime_quotes_now_zt_new_kc_open_copy = realtime_quotes_now_zt_new_kc_open.copy()

    if now_date_time > target_time_09_31:
        logger.error("当期日期代码无开盘数据:{},{}", str_day, kc_one.symbol)
        realtime_quotes_now_zt_new_kc_open_copy.loc[:, 'no_open_data'] = True
    else:
        realtime_quotes_now_zt_new_kc_open_copy.loc[:, 'no_open_data'] = False
        #

    realtime_quotes_now_zt_new_kc_open_copy.loc[:, 'yesterday_high_chg'] = kc_one.yesterday_high_chg
    realtime_quotes_now_zt_new_kc_open_copy.loc[:, 'today_chg'] = kc_one.chg
    realtime_quotes_now_zt_new_kc_open_copy.loc[:, 'str_day'] = str_day

    realtime_quotes_now_zt_new_kc_open_copy = handle_init_real_time_quotes_data(
        realtime_quotes_now_zt_new_kc_open_copy.copy(),
        str_now_date,
        1)
    realtime_quotes_now_zt_new_kc_open_copy = realtime_quotes_now_zt_new_kc_open_copy[
        realtime_quotes_now_zt_new_kc_open_field]

    # 将日期数值转换为日期时间格式
    realtime_quotes_now_zt_new_kc_open_copy['list_date_01'] = pd.to_datetime(
        realtime_quotes_now_zt_new_kc_open_copy['list_date'],
        format='%Y%m%d')
    # 将日期字符串转换为日期时间格式
    realtime_quotes_now_zt_new_kc_open_copy['str_day_01'] = pd.to_datetime(
        realtime_quotes_now_zt_new_kc_open_copy['str_day'],
        format='%Y-%m-%d')

    # 计算日期差值 距离现在上市时间
    realtime_quotes_now_zt_new_kc_open_copy['diff_days'] = realtime_quotes_now_zt_new_kc_open_copy.apply(
        lambda row: (row['str_day_01'] - row['list_date_01']).days, axis=1)

    del realtime_quotes_now_zt_new_kc_open_copy['str_day_01']
    del realtime_quotes_now_zt_new_kc_open_copy['list_date_01']

    deal_days = k_line_common_service_api.get_deal_days(str_day, kc_one.symbol)

    realtime_quotes_now_zt_new_kc_open_copy['deal_days'] = deal_days

    mongodb_util.save_mongo(realtime_quotes_now_zt_new_kc_open_copy, 'realtime_quotes_now_zt_new_kc_open')


# 同步当前day整天数据
def one_symbol_day_data(str_day, kc_one, realtime_quotes_now_kc):
    realtime_quotes_now_kc.loc[
        realtime_quotes_now_kc['symbol'] == kc_one.symbol, 'industry'] = kc_one.industry

    realtime_quotes_now_kc['str_day'] = str_day

    query_remove = {"str_day": str_day, "symbol": kc_one.symbol}

    success02 = mongodb_util.remove_data(query_remove, 'realtime_quotes_now_zt_new_kc').acknowledged
    if success02:
        mongodb_util.insert_mongo(realtime_quotes_now_kc, 'realtime_quotes_now_zt_new_kc')


def handle_init_real_time_quotes_data(real_time_quotes_now, str_now_date, number):
    #  exclude b symbol
    real_time_quotes_now = common_service_fun_api.exclude_b_symbol(real_time_quotes_now.copy())
    #  classification symbol
    real_time_quotes_now = common_service_fun_api.classify_symbol(real_time_quotes_now.copy())
    #  fix industry
    real_time_quotes_now = fix_industry_data(real_time_quotes_now.copy())
    #  calculate parameter
    real_time_quotes_now = data_init_api.calculate_parameter_factor(real_time_quotes_now.copy())

    real_time_quotes_now = real_time_quotes_now.loc[real_time_quotes_now['amount'] != 0]
    real_time_quotes_now['str_now_date'] = str_now_date
    real_time_quotes_now['number'] = number
    return real_time_quotes_now


# 同步当天所有开盘数据
def sync_one_day_open_data(str_day):
    realtime_quotes_db_name = 'realtime_quotes_now_' + str_day

    number = realtime_quotes_now_min_number_sync(realtime_quotes_db_name, 'number', str_day, "000001")

    query = {"number": number}
    db = get_db(str_day)
    realtime_quotes_now_list = db.find_query_data(realtime_quotes_db_name, query)

    realtime_quotes_now_one = realtime_quotes_now_list.iloc[0]
    str_now_date = realtime_quotes_now_one['_id']
    str_now_date = str_now_date[7:26]

    now_date = date_handle_util.str_to_date(str_now_date, "%Y-%m-%d %H:%M:%S")
    now_date_time = now_date.time()

    target_time_09_31 = time(9, 31)
    if now_date_time >= target_time_09_31:
        return

    realtime_quotes_now_list['str_day'] = str_day

    realtime_quotes_now_list = handle_init_real_time_quotes_data(
        realtime_quotes_now_list.copy(),
        str_now_date, number)

    mongodb_util.insert_mongo(realtime_quotes_now_list, 'realtime_quotes_now_open')

    logger.info("同步str_day:{}开盘数据", str_day)


def realtime_quotes_now_min_number_sync(db_name, field, str_day, symbol):
    db = get_db(str_day)
    query = {'symbol': symbol}
    df = db.ascend_query(query, db_name, field, 1)
    if df is None or df.shape[0] == 0:
        return 1
    else:
        return list(df[field])[0]


def realtime_quotes_now_zt_new_kc_open_sync():
    realtime_quotes_now_zt_new_kc_open = mongodb_util.find_all_data('realtime_quotes_now_zt_new_kc_open')
    realtime_quotes_now_zt_new_kc_open = realtime_quotes_now_zt_new_kc_open[realtime_quotes_now_zt_new_kc_open_field]
    realtime_quotes_now_zt_new_kc_open = realtime_quotes_now_zt_new_kc_open.sort_values(by=['str_day'], ascending=False)

    mongodb_util.insert_mongo(realtime_quotes_now_zt_new_kc_open, 'realtime_quotes_now_zt_new_kc_open_copy')


#     query = {'$and': [{"_id": {'$lte': str_end}}, {"_id": {'$gte': '2022-04-25'}}]}
def sync_all_high_chg_data(str_end):
    query = {'$and': [{"_id": {'$lte': str_end}}, {"_id": {'$gte': '2024-03-01'}}]}
    trade_date_list = mongodb_util.find_query_data('trade_date_list', query)
    trade_date_list = trade_date_list.sort_values(by=['trade_date'], ascending=False)
    for date_one in trade_date_list.itertuples():
        try:
            str_day = date_one.trade_date
            sync_all_kc_zt_data(str_day, None)
        except BaseException as e:
            logger.error("发生异常:{},{}", str_day, e)


def fix_miss_data(str_end):
    query = {'$and': [{"_id": {'$lte': str_end}}, {"_id": {'$gte': '2023-12-05'}}]}
    trade_date_list = mongodb_util.find_query_data('trade_date_list', query)
    trade_date_list = trade_date_list.sort_values(by=['trade_date'], ascending=False)
    for date_one in trade_date_list.itertuples():
        try:
            str_day = date_one.trade_date
            query = {"str_day": str_day, "miss_out": True}
            stock_high_chg_pool_df = mongodb_util.find_query_data('stock_high_chg_pool', query)
            if data_frame_util.is_empty(stock_high_chg_pool_df):
                continue
            miss_symbol_list = list(stock_high_chg_pool_df['symbol'])
            sync_all_kc_zt_data(str_day, miss_symbol_list)
        except BaseException as e:
            logger.error("发生异常:{},{}", str_day, e)


if __name__ == '__main__':
    sync_all_kc_zt_data('2024-05-17', None)
    # sync_all_kc_zt_data('2023-08-16')
    # sync_all_kc_zt_data('2023-07-07')
    # realtime_quotes_now_zt_new_kc_open_sync()
    # hui_ce_all('2023-06-16')
    # fix_diff_day()
    # sync_all_kc_zt_data('2023-06-30')

    # sync_all_kc_zt_data('2023-07-05')
    # sync_one_day_open_data('2023-07-05')
    # sync_all_kc_zt_data('2023-06-30')
    # sync_one_day_open_data('2023-05-31')
    # hui_ce_all('2023-03-16')
    # sync_all_kc_zt_data('2023-06-28')
    # sync_all_kc_zt_data('2023-05-12')
    # sync_all_kc_zt_data('2023-05-15')
    # sync_all_kc_zt_data('2023-05-16')
    # sync_all_kc_zt_data('2023-05-17')
    # sync_all_kc_zt_data('2023-05-10')



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)
from datetime import datetime
import mns_common.component.self_choose.black_list_service_api as black_list_service_api
import mns_scheduler.finance.finance_common_api as finance_common_api
from loguru import logger
import mns_common.constant.db_name_constant as db_name_constant
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api
import mns_common.component.common_service_fun_api as common_service_fun_api
import pandas as pd

# 1.无保留意见/标准报告：报告没问题。（没有发现造假，但也不能保证为真）
#
# 2.带强调事项段的无保留意见：报告没问题，但是有亏损获对其可持续经营有重大疑虑（可能造假，至少是在粉饰报表）
#
# 3.保留意见报告：有问题，财务造假
#
# 4.否定意见报告：有很大问题
#
# 5.无法表示意见报告：不让查
#

mongodb_util = MongodbUtil('27017')
# 审核标准意见
OPINION_TYPE = "标准无保留意见"

# 利润为负的时候最小营业收入 主板 3.2亿
MIN_INCOME_MAIN = 320000000
# 利润为负的时候最小营业收入 科创 创业 1.2亿
MIN_INCOME_SUB = 120000000
# 最大负债比
MAX_LIABILITY_RATIO = 90
# 负载超过90%时候最小净资产
MIN_NET_ASSET = 1000000000
# 排除校验负债比的行业
EXCLUDE_INDUSTRY = ['保险', '银行', '证券']

# 最迟出报告的天数
LATE_REPORT_DAYS = 3


#### 退市新规 ####
# 1 股价类:连续20个交易日估价低于1元
# 2 市值类: 主板小于5亿、创业板3亿
# 3 财务类: (1) 利润总额 净利润 扣非净利润三者最小值为负 且营业收入小于3亿 创业板营业收入小于1元
#          (2) 资不抵债

# 财报审核
def financial_report_check(new_report_df, period_time, period, report_type):
    if period == 4:
        # 年报异常审核
        year_report_exception_check(new_report_df, period_time, report_type)
    # 负债过高
    liability_ratio_check(report_type, new_report_df, period_time)


# 年报审核异常
def year_report_exception_check(new_report_df, period_time, report_type):
    new_report_one_df = new_report_df.loc[new_report_df['REPORT_DATE'] == period_time]
    # 审核意见
    opinion_type = list(new_report_one_df['OPINION_TYPE'])[0]
    symbol = list(new_report_one_df['SECURITY_CODE'])[0]
    name = list(new_report_one_df['SECURITY_NAME_ABBR'])[0]
    now_date = datetime.now()
    str_day = now_date.strftime('%Y-%m-%d')
    str_now_date = now_date.strftime('%Y-%m-%d %H:%M:%S')

    # 年报有问题
    if opinion_type != OPINION_TYPE:
        id_key = symbol + "_" + period_time + "_" + black_list_service_api.FINANCIAL_PROBLEM_ANNUAL_REPORT
        black_list_service_api.save_black_stock(id_key,
                                                symbol,
                                                name,
                                                str_day,
                                                str_now_date,
                                                '年报审计有问题:' + "[" + str(opinion_type) + "]",
                                                '年报审计有问题',
                                                '',
                                                black_list_service_api.FINANCIAL_PROBLEM_ANNUAL_REPORT)

    if report_type == db_name_constant.EM_STOCK_PROFIT:
        # 利润总额  净利润 扣除非经常性损益后的净利润  三者最小为负
        # 利润总额
        total_profit = list(new_report_one_df['TOTAL_PROFIT'])[0]
        #  净利润
        net_profit = list(new_report_one_df['NETPROFIT'])[0]
        # 营业利润
        operate_profit = list(new_report_one_df['OPERATE_PROFIT'])[0]
        # 持续经营净利润
        continued_profit = list(new_report_one_df['CONTINUED_NETPROFIT'])[0]
        # 归属于母公司股东的净利润
        parent_profit = list(new_report_one_df['PARENT_NETPROFIT'])[0]
        # 扣除非经常性损益后的净利润
        deduct_parent_profit = list(new_report_one_df['DEDUCT_PARENT_NETPROFIT'])[0]
        # 营业总收入
        total_operate_income = list(new_report_one_df['TOTAL_OPERATE_INCOME'])[0]
        if total_operate_income == 0:
            #  营业收入
            total_operate_income = list(new_report_one_df['OPERATE_INCOME'])[0]

            # 最小利润收入
        min_profit = min(total_profit, net_profit, operate_profit,
                         continued_profit, parent_profit, deduct_parent_profit)
        if min_profit < 0:

            classification = common_service_fun_api.classify_symbol_one(symbol)
            if ((classification in ['S', 'H'] and total_operate_income < MIN_INCOME_MAIN)
                    | (classification in ['K', 'C'] and total_operate_income < MIN_INCOME_SUB)):
                id_key = symbol + "_" + period_time + "_" + black_list_service_api.FINANCIAL_PROBLEM_PROFIT
                min_profit = round(min_profit / common_service_fun_api.TEN_THOUSAND, 1)
                total_operate_income = round(total_operate_income / common_service_fun_api.HUNDRED_MILLION, 1)

                black_list_service_api.save_black_stock(id_key,
                                                        symbol,
                                                        name,
                                                        str_day,
                                                        str_now_date,
                                                        '年报:利润:' + '[' + str(min_profit) + '万]' + '收入:' + str(
                                                            total_operate_income) + '[' + '亿元]--' + '触发退市风险',
                                                        '年报:利润:' + '[' + str(min_profit) + '万]' + '收入:' + str(
                                                            total_operate_income) + '[' + '亿元]--' + '触发退市风险',
                                                        '',
                                                        black_list_service_api.FINANCIAL_PROBLEM_PROFIT)


# 负债比校验
def liability_ratio_check(report_type, new_report_df, period_time):
    if report_type == db_name_constant.EM_STOCK_ASSET_LIABILITY:
        new_report_df = new_report_df.sort_values(by=['REPORT_DATE'], ascending=False)
        new_report_one_df = new_report_df.iloc[0:1]
        # 负债比
        liability_ratio = list(new_report_one_df['liability_ratio'])[0]
        # 净资产
        net_asset = round(list(new_report_one_df['TOTAL_ASSETS'])[0] - list(new_report_one_df['TOTAL_LIABILITIES'])[0],
                          2)

        symbol = list(new_report_one_df['SECURITY_CODE'])[0]
        name = list(new_report_one_df['SECURITY_NAME_ABBR'])[0]
        now_date = datetime.now()
        str_day = now_date.strftime('%Y-%m-%d')
        str_now_date = now_date.strftime('%Y-%m-%d %H:%M:%S')
        id_key = symbol + "_" + period_time + "_" + black_list_service_api.FINANCIAL_PROBLEM_DEBT

        query_company = {'_id': symbol, 'industry': {'$in': EXCLUDE_INDUSTRY}}
        if mongodb_util.exist_data_query(db_name_constant.COMPANY_INFO, query_company):
            return None

        if liability_ratio >= MAX_LIABILITY_RATIO:
            black_list_service_api.save_black_stock(id_key,
                                                    symbol,
                                                    name,
                                                    str_day,
                                                    str_now_date,
                                                    '负债过高:' + "[" + str(
                                                        liability_ratio) + "]" + "," + "净资产:" + str(round(
                                                        net_asset / common_service_fun_api.HUNDRED_MILLION, 0)) + "亿",
                                                    '负债过高:' + "[" + str(liability_ratio) + "]",
                                                    '',
                                                    black_list_service_api.FINANCIAL_PROBLEM_DEBT)


# 未出财报
def un_report_check(sync_time, now_year, period, period_time):
    un_report_asset_df = finance_common_api.find_un_report_symbol(period_time,
                                                                  db_name_constant.EM_STOCK_ASSET_LIABILITY)
    un_report_profit_df = finance_common_api.find_un_report_symbol(period_time,
                                                                   db_name_constant.EM_STOCK_PROFIT)
    un_report_df = pd.concat([un_report_asset_df, un_report_profit_df])
    if period == 4 or period == 1:
        last_report_day = str(now_year) + "-05-01"
    elif period == 2:
        last_report_day = str(now_year) + "-07-01"
    elif period == 3:
        last_report_day = str(now_year) + "-10-01"
    max_report_day = trade_date_common_service_api.get_before_trade_date(last_report_day, LATE_REPORT_DAYS)
    if max_report_day >= sync_time:

        for un_asset_one in un_report_df.itertuples():
            symbol = un_asset_one.symbol
            id_key = symbol + "_" + period_time + "_" + black_list_service_api.FINANCIAL_PROBLEM_NOT_REPORT
            name = un_asset_one.name
            now_date = datetime.now()
            str_day = now_date.strftime('%Y-%m-%d')
            try:

                black_list_service_api.save_black_stock(id_key,
                                                        symbol,
                                                        name,
                                                        str_day,
                                                        sync_time,
                                                        '未出财报',
                                                        '未出财报',
                                                        '',
                                                        black_list_service_api.FINANCIAL_PROBLEM_NOT_REPORT)
            except Exception as e:
                logger.error("更新未出报告异常:{},{},{}", symbol, period_time, e)
